@@ -16,7 +16,7 @@ class name for the list object is FS::List.
 A list object provides methods for managing a list of FS::Hash objects
 of the same class -- storing, retrieving and searching on them.
 
-Listclass shares most of the API with the Hash class.
+List class shares most of the API with the Hash class.
 
 Here is the list of all List methods (alphabetically):
 
@@ -33,7 +33,7 @@ use XAO::Objects;
 use base XAO::Objects->load(objname => 'FS::Glue');
 
 use vars qw($VERSION);
-$VERSION=(0+sprintf('%u.%03u',(q$Id: List.pm,v 2.1 2005/01/14 00:23:54 am Exp $ =~ /\s(\d+)\.(\d+)\s/))) || die "Bad VERSION";
+$VERSION=(0+sprintf('%u.%03u',(q$Id: List.pm,v 2.10 2009/03/16 21:28:54 am Exp $ =~ /\s(\d+)\.(\d+)\s/))) || die "Bad VERSION";
 
 ###############################################################################
 
@@ -156,8 +156,8 @@ sub describe ($;$) {
     my $self=shift;
     return {
         type    => @_ ? 'hash' : 'list',
-        class   => $$self->{class_name},
-        key     => $$self->{key_name},
+        class   => $$self->{'class_name'},
+        key     => $$self->{'key_name'},
     };
 }
 
@@ -239,7 +239,7 @@ that list can store.
 
 sub get_new ($) {
     my $self=shift;
-    $self->_glue->new(objname => $$self->{class_name});
+    $self->_glue->new(objname => $$self->{'class_name'});
 }
 
 ###############################################################################
@@ -255,6 +255,34 @@ from.
 
 ###############################################################################
 
+=item key_charset
+
+Returns key charset for the given list. Default is 'binary'.
+
+=cut
+
+sub key_charset ($) {
+    my $self=shift;
+    return $$self->{'key_charset'} ||
+        throw $self "key_charset - not defined (internal logic problem, $$self->{'class_name'})";
+}
+
+###############################################################################
+
+=item key_format
+
+Returns key format for the given list. Default is '<$RANDOM$>'.
+
+=cut
+
+sub key_format ($) {
+    my $self=shift;
+    return $$self->{'key_format'} ||
+        throw $self "key_format - not defined (internal logic problem, $$self->{'class_name'})";
+}
+
+###############################################################################
+
 =item key_length
 
 Returns key length for the given list. Default is 30.
@@ -263,7 +291,8 @@ Returns key length for the given list. Default is 30.
 
 sub key_length ($) {
     my $self=shift;
-    return $$self->{key_length} || 30;
+    return $$self->{'key_length'} ||
+        throw $self "key_length - not defined (internal logic problem, $$self->{'class_name'})";
 }
 
 ###############################################################################
@@ -277,7 +306,7 @@ Returns unsorted list of all keys for all objects stored in that list.
 sub keys ($) {
     my $self=shift;
 
-    if($$self->{detached}) {
+    if($$self->{'detached'}) {
         $self->throw("keys - is not implemented yet for detached mode");
     } else {
         @{$self->_list_keys()};
@@ -382,7 +411,7 @@ sub put ($$;$) {
     else {
         ($name,$value)=@_;
         $self->check_name($name) ||
-            throw $self "put - wrong name (name=$name, class=$$self->{class_name}";
+            throw $self "put - wrong name (name=$name, class=$$self->{class_name})";
     }
 
     if($$self->{detached}) {
@@ -394,6 +423,53 @@ sub put ($$;$) {
 
 ###############################################################################
 
+=item scan (@)
+
+A wrapper for search() method that makes it easier to search in large
+collections of data. Will use 'offset' and 'limit' options to search in
+multiple iterations and will use callbacks to process results.
+
+Parameters are:
+
+  block_size        => required, how many records to retrieve at once
+  search_query      => optional array reference, see below in search() method
+  search_options    => required, must include at least an 'orderby' option
+  call_before       => optional, called before the start
+  call_block        => optional, called with the full retrieved data block
+  call_row          => optional, called for each row of the data
+  call_after        => optional, called after the scanning is done
+
+All callbacks get at least two arguments: the reference to the list or
+collection being scanned, and the reference to the parameters passed
+to scan(). Call_block also gets a reference to the block, and its
+zero-based offset. Call_row gets the result (either an array ref or a
+scalar, depending on 'result' option), its zero-based offset.
+
+Scan() does not return anything, so at least one callback is necessary.
+
+Example:
+
+    $list->scan(
+        block_size      => 10000,
+        search_options  => {
+            orderby => 'item_code',
+            result  => [ 'item_code','cat_page' ],
+        },
+        call_block      => sub {
+            my ($l,$a,$block,$offset)=@_;
+            dprint "..$offset";
+            foreach my $row (@$block) {
+                #...
+            }
+        },
+    );
+
+=cut
+
+# Implemented in Glue
+
+###############################################################################
+
 =item search (@)
 
 Returns a reference to the list of IDs of objects corresponding to the
@@ -415,7 +491,7 @@ translated to the same database query:
 
  my $r=$list->search('name', 'wq', [ 'big', 'ugly' ]);
 
- my $r=$list->search([ 'name', 'wq', 'big' ], 
+ my $r=$list->search([ 'name', 'wq', 'big' ],
                      'or',
                      [ 'name', 'wq', 'ugly' ]);
 
@@ -439,7 +515,7 @@ Example:
                          'and',
                          [ '../gross_premium', 'lt', 1000 ]);
 
-Sometimes it might be necessary to check is a pair of objects inside of
+Sometimes it might be necessary to check if a pair of objects inside of
 some container have specific properties. This can be achieved with
 instance specificators:
 
@@ -537,18 +613,12 @@ True if property contains the given word completely. For example
 ['name', 'wq', 'ann'] would match 'Ann Peters' and 'Marie Ann', but
 would not match 'Annette'.
 
-For best performance please make this kind of search only on fields of
-type 'words' -- in that case the search is performed by dictionary and
-is very fast.
-
 =item ws
 
 True if property contains a word that starts with the given
 text. For example ['name', 'ws', 'an'] would match 'Andrew'
 and 'Marie Ann', but 'Joann' would not match.
 
-Works best on fields of type 'words'.
-
 =back
 
 Logical operators:
@@ -624,7 +694,7 @@ field. Example:
 
  my $color_ids=$products->search('category_id', 'eq', 123, {
                                     'distinct' => 'color'
-                                }); 
+                                });
 
 =item debug
 
@@ -666,65 +736,75 @@ underlying database does not support this feature.
                                  'limit' => 100
                             });
 
+=item offset
+
+Indicates that you are only interested in results starting at the
+'offset' position in the resulting set. In combination with 'limit' this
+can be used to page through large result sets.
+
 =item result
 
-Note: [Not completely implemented yet]
+This is a very powerful option that can significantly decrease your
+database load if used properly.
 
-Be default search() method returns a reference to an array of object
-keys. Result options allows you to alter that behavior.
+By default search() method returns a reference to an array of object
+keys. The 'result' option allows you to alter that behavior and receive
+database values in the same query, avoiding an extra step of retrieving
+the object and calling its get() method.
 
-Generally you can pass single description of return value or multiple
-descriptions as an array reference. In the first case what you get then
-is array of scalars, in the second case -- you get an array of arrays of
-scalars.
+Note, that if you're going to use only a portion of the results it may
+be faster to get only IDs in the usual manner and then load the data you
+need. Loading multiple fields of data on more objects than you actually
+need may be significantly slower.
 
-Description of return value can be a scalar -- in that case it is simply
-a name of field in the database; or it can be a hash reference. For hash
-reference the only required field is 'type', that determines type of the
-result. Other parameters in the hash depend on the specific type.
+You can pass a single description of a return field or multiple descriptions
+as an array reference. Regardless of how many fields you request if
+'result' option is used you always get a reference to an array of
+arrays.
 
-Recognized types are:
+The returning array rows may contain more records than you requested,
+but it is guaranteed that they will contain at least as many as
+requested and in the same order as requested. It's best to ignore the
+extra fields should you get any, their presense or content are not
+guaranteed.
+
+In addition to usual field names some special names are supported:
 
 =over
 
-=item count
+=item #connector
 
-No other parameters, return number of would-be results for the
-search. Resulting array will have only one row if 'count' is used.
+Returns the collection key for "parent" object (the containing the list
+being searched on). The value of the key can be used to pull the parent
+object from its class based collection. The typical use is to identify
+different object parents for future processing without pulling any of
+their data -- allows for faster searching without any table joins.
 
-=item key
+=item #container_key
 
-Returns object ID, just the same as would be returned by default.
+Returns the value of the key in the list you're searching on.
 
-=item sum
+=item #collection_key
 
-Returns arithmetic sum of all 'name' fields in the resulting set.
+Returns the value of the collection key for the object
+(collection-unique, can be used to pull this object from a collection
+rather than a specific list).
+
+=item #id
+
+Returns collection key if the search is being done on a collection, and
+a list key if on a list. In other words, returns the same ID that would
+have been returned without the 'result' option.
 
 =back
 
 Examples:
 
- my $rr=$data->search('last_name','cs','smit', {
-                    orderby => 'last_name',
-                    result => [qw(id last_name first_name age)]
-                });
-
- my $rr=$data->search({
-                    result => {
-                        type    => 'count',
-                    },
-                 });
- my $count=$rr->[0];
-
- my $rr=$data->search({
-                    result => [ {
-                        type    => 'count',
-                    }, {
-                        type    => 'sum',
-                        name    => 'gross_rev',
-                    }
-                ] });
- my ($count,$sum)=@{$rr->[0]};
+ my $rr=$data->search(
+    'last_name','cs','smit', {
+    orderby => 'last_name',
+    result => [qw(#container_key last_name first_name age)]
+ });
 
 =back
 
@@ -735,7 +815,6 @@ rows in your code anyway.
 As a degraded case of search it is safe to pass nothing or just options
 to select everything in the given list. Examples:
 
- ##
  # These two lines are roughly equivalent. Note that you get an array
  # reference in the first line and an array itself in the second.
  #
@@ -743,11 +822,14 @@ to select everything in the given list. Examples:
 
  my @keys=$products->keys();
 
- ##
  # This is the way to get all the keys ordered by price.
  #
  my $keys=$products->search({ orderby => 'price' });
 
+ # Getting name and surname for all records
+ #
+ my $data=$customers->search({ result => [qw(name surname)] });
+
 =cut
 
 sub search ($@) {
